#include "seznam.h"
#include <iostream>

void vytvor_seznam(Seznam& s)
{
	s.hlava = s.zarazka = new Prvek{ NULA, nullptr };
	s.velikost = 0;
}

void vloz_na_zacatek(Seznam& s, T co)
{
	s.hlava = new Prvek{ co, s.hlava };
	s.velikost++;
}

void vloz_na_konec(Seznam& s, T co)
{
	s.zarazka->data = co;
	s.zarazka->dalsi = new Prvek{ NULA, nullptr };
	s.zarazka = s.zarazka->dalsi;
	s.velikost++;
}

void odstran_prvni(Seznam& s)
{
	if (s.hlava == s.zarazka)
		return; // seznam byl prazdny
	auto pom = s.hlava;
	s.hlava = s.hlava->dalsi;
	delete pom;
	s.velikost--;
}

bool prazdny(Seznam& s)
{
	//return s.hlava == s.zarazka;
	return !s.velikost;
}

size_t velikost_seznamu(Seznam& s)
{
	return s.velikost;
}

void vyprazdni(Seznam& s)
{
	while (!prazdny(s))
	{
		odstran_prvni(s);
	}
}

void zrus(Seznam& s)
{
	vyprazdni(s);
	delete s.hlava;
	s.hlava = s.zarazka = nullptr;
}


Prvek* najdi(Seznam& s, T hodnota)
{
	Prvek* pom = s.hlava;
	s.zarazka->data = hodnota; // pomucka, abychom to VZDY nasli
	while (pom) // dokud je kam ukazovat (vcetne zarazky)
	{
		if (pom->data == hodnota) // hura, nalezeno
		{
			s.zarazka->data = NULA; // zarazku dame do puvodniho stavu
			if (pom == s.zarazka) // nasli jsme to az na fiktivnim konci?
				return nullptr; // nenalezeno
			else
				return pom;
		}
		pom = pom->dalsi; // na toto jsem 7. 11. 2022 zapomnela (byl by to nekonecny cyklus!)
	}
}

void vloz_za_prvek(Seznam& s, T hledane, T pridane) // lepsi nazev: "vloz_za_hodnotu"
{
	Prvek* nalezeno;
	nalezeno = najdi(s,hledane);
	if (!nalezeno) // "najdi" vratilo nullptr (tj. nenalezeno)
		vloz_na_konec(s, pridane); // vlozime na konec
	else
		nalezeno->dalsi = new Prvek{pridane, nalezeno->dalsi}; // vlozime "doprostred"
	s.velikost++; // aktualizujeme pocet prvku
}

/* void odstran_hodnotu(Seznam& s, T hodnota)
{
	Prvek* nalezen = najdi(s,hodnota);
	if (!nalezen)
		return; // prvek v seznamu neni, nemusime nic odstranovat
	Prvek* dalsi_prvek = nalezen; // !!! puvodne uvnitr while (to by pak neslo dealokovat!)
	while (nalezen->dalsi) // nemame ukazatel na predchoziho => nakopirujeme postupne zezadu dopredu
	{
		dalsi_prvek = nalezen->dalsi; // prvni prvek vpravo
		// prekopirovani hodnot "dopredu":
		nalezen->data = dalsi_prvek->data;
		nalezen->dalsi = dalsi_prvek->dalsi;
		if (nalezen->data == NULA) // predposledni prvek je nova zarazka! Upozornit 14. 11. 2022
			s.zarazka = nalezen;
		// a posun vpravo:
		nalezen = dalsi_prvek;
	}
	// s.zarazka = nalezen; // zde je na nastaveni nove zarazky uz pozde!
	delete dalsi_prvek; // dealokovat starou (nyni "druhou") zarazku
	s.velikost--;
} */

Prvek* minimum(Seznam& s)
{
	Prvek *min_prvek, *pom; // dva pomocne ukazatele (na tabuli byla chyba u "pom")
	if (prazdny(s))
		return nullptr; // prazdny seznam nema minimum
	min_prvek = pom = s.hlava; // zaciname 1. prvkem
	while (pom != s.zarazka) // dokud nejsme na konci
	{
		if (pom->data < min_prvek->data) // nasli jsme mensi hodnotu
			min_prvek = pom; // novy prvek, kde je minimum
		pom = pom->dalsi; // posuneme se dal
	}
	return min_prvek;
}

void vypis(Seznam& s)
{
	auto pom = s.hlava;
	while (pom != s.zarazka)
	{
		std::cout << pom->data << " "; // mezi hodnotami vypiseme mezeru
		pom = pom->dalsi;
	}
	std::cout << std::endl; // nakonec odradkujeme
}